@@ -2,10 +2,8 @@
 """
 Pure Python GeoIP API
 
-The API is based on MaxMind's C-based Python API, but the code itself is
-ported from the Pure PHP GeoIP API by Jim Winstead and Hans Lellelid.
-
 @author: Jennifer Ennis <zaylea@gmail.com>
+@author: William Tisäter <william@defunct.cc>
 
 @license: Copyright(C) 2004 MaxMind LLC
 
@@ -24,14 +22,19 @@ along with this program.  If not, see <http://www.gnu.org/licenses/lgpl.txt>.
 """
 
 import os
-import math
 import socket
-import mmap
 import codecs
+from math import floor
 from threading import Lock
 
+try:
+    import mmap
+except ImportError:  # pragma: no cover
+    mmap = None
+
 try:
     from StringIO import StringIO
+    range = xrange  # Use xrange for Python 2
 except ImportError:
     from io import StringIO, BytesIO
 
@@ -51,34 +54,36 @@ class GeoIPError(Exception):
     pass
 
 
-class GeoIPMetaclass(type):
-    def __new__(cls, *args, **kwargs):
-        """
-        Singleton method to gets an instance without reparsing the db. Unique
-        instances are instantiated based on the filename of the db. Flags are
-        ignored for this, i.e. if you initialize one with STANDARD
-        flag (default) and then try later to initialize with MEMORY_CACHE, it
-        will still return the STANDARD one.
-        """
-        if not hasattr(cls, '_instances'):
-            cls._instances = {}
+class _GeoIPMetaclass(type):
+    _instances = {}
+    _instance_lock = Lock()
 
+    def __call__(cls, *args, **kwargs):
+        """ Singleton method to gets an instance without reparsing
+        the database, the filename is being used as cache key.
+        """
         if len(args) > 0:
             filename = args[0]
         elif 'filename' in kwargs:
             filename = kwargs['filename']
+        else:
+            return None
 
+        if not kwargs.get('cache', True):
+            return super(_GeoIPMetaclass, cls).__call__(*args, **kwargs)
+
+        cls._instance_lock.acquire()
         if filename not in cls._instances:
-            cls._instances[filename] = type.__new__(cls, *args, **kwargs)
+            cls._instances[filename] = super(_GeoIPMetaclass, cls).__call__(*args, **kwargs)
+        cls._instance_lock.release()
 
         return cls._instances[filename]
 
 
-GeoIPBase = GeoIPMetaclass('GeoIPBase', (object,), {})
+class GeoIP(object):
+    __metaclass__ = _GeoIPMetaclass
 
-
-class GeoIP(GeoIPBase):
-    def __init__(self, filename, flags=0):
+    def __init__(self, filename, flags=0, cache=True):
         """
         Initialize the class.
 
@@ -89,29 +94,48 @@ class GeoIP(GeoIPBase):
             MEMORY_CACHE (preload the whole file into memory) and
             MMAP_CACHE (access the file via mmap).
         @type flags: int
+        @param cache: Used in tests to skip instance caching
+        @type cache: bool
         """
-        self._filename = filename
         self._flags = flags
+        self._netmask = None
+
+        if self._flags & const.MMAP_CACHE and mmap is None:  # pragma: no cover
+            import warnings
+            warnings.warn("MMAP_CACHE cannot be used without a mmap module")
+            self._flags &= ~const.MMAP_CACHE
 
         if self._flags & const.MMAP_CACHE:
-            f = open(filename, 'rb')
+            f = codecs.open(filename, 'rb', ENCODING)
             access = mmap.ACCESS_READ
-            self._filehandle = mmap.mmap(f.fileno(), 0, access=access)
+            self._fp = mmap.mmap(f.fileno(), 0, access=access)
+            self._type = 'MMAP_CACHE'
             f.close()
-
         elif self._flags & const.MEMORY_CACHE:
-            f = open(filename, 'rb')
-            self._memoryBuffer = f.read()
-            iohandle = BytesIO if PY3 else StringIO
-            self._filehandle = iohandle(self._memoryBuffer)
+            f = codecs.open(filename, 'rb', ENCODING)
+            self._memory = f.read()
+            self._fp = self._str_to_fp(self._memory)
+            self._type = 'MEMORY_CACHE'
             f.close()
-
         else:
-            self._filehandle = codecs.open(filename, 'rb', ENCODING)
+            self._fp = codecs.open(filename, 'rb', ENCODING)
+            self._type = 'STANDARD'
 
         self._lock = Lock()
         self._setup_segments()
 
+    @classmethod
+    def _str_to_fp(cls, data):
+        """
+        Convert bytes data to file handle object
+
+        @param data: string data
+        @type data: str
+        @return: file handle object
+        @rtype: StringIO or BytesIO
+        """
+        return BytesIO(bytearray(data, ENCODING)) if PY3 else StringIO(data)
+
     def _setup_segments(self):
         """
         Parses the database file to determine what kind of database is
@@ -138,12 +162,12 @@ class GeoIP(GeoIPBase):
         self._databaseSegments = const.COUNTRY_BEGIN
 
         self._lock.acquire()
-        filepos = self._filehandle.tell()
-        self._filehandle.seek(-3, os.SEEK_END)
+        filepos = self._fp.tell()
+        self._fp.seek(-3, os.SEEK_END)
 
         for i in range(const.STRUCTURE_INFO_MAX_SIZE):
             chars = chr(255) * 3
-            delim = self._filehandle.read(3)
+            delim = self._fp.read(3)
 
             if PY3 and type(delim) is bytes:
                 delim = delim.decode(ENCODING)
@@ -154,11 +178,11 @@ class GeoIP(GeoIPBase):
                     delim = delim.decode(ENCODING)
 
             if delim == chars:
-                byte = self._filehandle.read(1)
+                byte = self._fp.read(1)
                 self._databaseType = ord(byte)
 
                 # Compatibility with databases from April 2003 and earlier
-                if (self._databaseType >= 106):
+                if self._databaseType >= 106:
                     self._databaseType -= 105
 
                 if self._databaseType == const.REGION_EDITION_REV0:
@@ -175,7 +199,7 @@ class GeoIP(GeoIPBase):
                                             const.ASNUM_EDITION,
                                             const.ASNUM_EDITION_V6):
                     self._databaseSegments = 0
-                    buf = self._filehandle.read(const.SEGMENT_RECORD_LENGTH)
+                    buf = self._fp.read(const.SEGMENT_RECORD_LENGTH)
 
                     if PY3 and type(buf) is bytes:
                         buf = buf.decode(ENCODING)
@@ -188,9 +212,9 @@ class GeoIP(GeoIPBase):
                         self._recordLength = const.ORG_RECORD_LENGTH
                 break
             else:
-                self._filehandle.seek(-4, os.SEEK_CUR)
+                self._fp.seek(-4, os.SEEK_CUR)
 
-        self._filehandle.seek(filepos, os.SEEK_SET)
+        self._fp.seek(filepos, os.SEEK_SET)
         self._lock.release()
 
     def _seek_country(self, ipnum):
@@ -211,13 +235,13 @@ class GeoIP(GeoIPBase):
                 if self._flags & const.MEMORY_CACHE:
                     startIndex = 2 * self._recordLength * offset
                     endIndex = startIndex + (2 * self._recordLength)
-                    buf = self._memoryBuffer[startIndex:endIndex]
+                    buf = self._memory[startIndex:endIndex]
                 else:
                     startIndex = 2 * self._recordLength * offset
                     readLength = 2 * self._recordLength
                     self._lock.acquire()
-                    self._filehandle.seek(startIndex, os.SEEK_SET)
-                    buf = self._filehandle.read(readLength)
+                    self._fp.seek(startIndex, os.SEEK_SET)
+                    buf = self._fp.read(readLength)
                     self._lock.release()
 
                 if PY3 and type(buf) is bytes:
@@ -230,13 +254,15 @@ class GeoIP(GeoIPBase):
                         x[i] += ord(byte) << (j * 8)
                 if ipnum & (1 << depth):
                     if x[1] >= self._databaseSegments:
+                        self._netmask = seek_depth - depth + 1
                         return x[1]
                     offset = x[1]
                 else:
                     if x[0] >= self._databaseSegments:
+                        self._netmask = seek_depth - depth + 1
                         return x[0]
                     offset = x[0]
-        except:
+        except (IndexError, UnicodeDecodeError):
             pass
 
         raise GeoIPError('Corrupt database')
@@ -255,8 +281,8 @@ class GeoIP(GeoIPBase):
 
         read_length = (2 * self._recordLength - 1) * self._databaseSegments
         self._lock.acquire()
-        self._filehandle.seek(seek_org + read_length, os.SEEK_SET)
-        buf = self._filehandle.read(const.MAX_ORG_RECORD_LENGTH)
+        self._fp.seek(seek_org + read_length, os.SEEK_SET)
+        buf = self._fp.read(const.MAX_ORG_RECORD_LENGTH)
         self._lock.release()
 
         if PY3 and type(buf) is bytes:
@@ -266,19 +292,18 @@ class GeoIP(GeoIPBase):
 
     def _get_region(self, ipnum):
         """
-        Seek and return the region info (dict containing country_code
-        and region_name).
+        Seek and return the region information.
 
         @param ipnum: Converted IP address
         @type ipnum: int
-        @return: dict containing country_code and region_name
+        @return: dict containing country_code and region_code
         @rtype: dict
         """
-        region = ''
-        country_code = ''
+        region_code = None
+        country_code = None
         seek_country = self._seek_country(ipnum)
 
-        def get_region_name(offset):
+        def get_region_code(offset):
             region1 = chr(offset // 26 + 65)
             region2 = chr(offset % 26 + 65)
             return ''.join([region1, region2])
@@ -287,7 +312,7 @@ class GeoIP(GeoIPBase):
             seek_region = seek_country - const.STATE_BEGIN_REV0
             if seek_region >= 1000:
                 country_code = 'US'
-                region = get_region_name(seek_region - 1000)
+                region_code = get_region_code(seek_region - 1000)
             else:
                 country_code = const.COUNTRY_CODES[seek_region]
         elif self._databaseType == const.REGION_EDITION_REV1:
@@ -296,20 +321,20 @@ class GeoIP(GeoIPBase):
                 pass
             elif seek_region < const.CANADA_OFFSET:
                 country_code = 'US'
-                region = get_region_name(seek_region - const.US_OFFSET)
+                region_code = get_region_code(seek_region - const.US_OFFSET)
             elif seek_region < const.WORLD_OFFSET:
                 country_code = 'CA'
-                region = get_region_name(seek_region - const.CANADA_OFFSET)
+                region_code = get_region_code(seek_region - const.CANADA_OFFSET)
             else:
                 index = (seek_region - const.WORLD_OFFSET) // const.FIPS_RANGE
                 if index in const.COUNTRY_CODES:
                     country_code = const.COUNTRY_CODES[index]
         elif self._databaseType in const.CITY_EDITIONS:
             rec = self._get_record(ipnum)
-            region = rec.get('region_name', '')
-            country_code = rec.get('country_code', '')
+            region_code = rec.get('region_code')
+            country_code = rec.get('country_code')
 
-        return {'country_code': country_code, 'region_name': region}
+        return {'country_code': country_code, 'region_code': region_code}
 
     def _get_record(self, ipnum):
         """
@@ -317,9 +342,9 @@ class GeoIP(GeoIPBase):
 
         @param ipnum: Converted IP address
         @type ipnum: int
-        @return: dict with country_code, country_code3, country_name,
-            region, city, postal_code, latitude, longitude,
-            dma_code, metro_code, area_code, region_name, time_zone
+        @return: dict with city, region_code, area_code, time_zone,
+            dma_code, metro_code, country_code3, latitude, postal_code,
+            longitude, country_code, country_name, continent
         @rtype: dict
         """
         seek_country = self._seek_country(ipnum)
@@ -328,8 +353,8 @@ class GeoIP(GeoIPBase):
 
         read_length = (2 * self._recordLength - 1) * self._databaseSegments
         self._lock.acquire()
-        self._filehandle.seek(seek_country + read_length, os.SEEK_SET)
-        buf = self._filehandle.read(const.FULL_RECORD_LENGTH)
+        self._fp.seek(seek_country + read_length, os.SEEK_SET)
+        buf = self._fp.read(const.FULL_RECORD_LENGTH)
         self._lock.release()
 
         if PY3 and type(buf) is bytes:
@@ -338,101 +363,91 @@ class GeoIP(GeoIPBase):
         record = {
             'dma_code': 0,
             'area_code': 0,
-            'metro_code': '',
-            'postal_code': ''
+            'metro_code': None,
+            'postal_code': None
         }
 
         latitude = 0
         longitude = 0
-        buf_pos = 0
 
-        # Get country
-        char = ord(buf[buf_pos])
+        char = ord(buf[0])
         record['country_code'] = const.COUNTRY_CODES[char]
         record['country_code3'] = const.COUNTRY_CODES3[char]
         record['country_name'] = const.COUNTRY_NAMES[char]
         record['continent'] = const.CONTINENT_NAMES[char]
 
-        buf_pos += 1
-        def get_data(buf, buf_pos):
-            offset = buf_pos
-            char = ord(buf[offset])
-            while (char != 0):
-                offset += 1
-                char = ord(buf[offset])
-            if offset > buf_pos:
-                return (offset, buf[buf_pos:offset])
-            return (offset, '')
+        def read_data(buf, pos):
+            cur = pos
+            while buf[cur] != '\0':
+                cur += 1
+            return cur, buf[pos:cur] if cur > pos else None
 
-        offset, record['region_name'] = get_data(buf, buf_pos)
-        offset, record['city'] = get_data(buf, offset + 1)
-        offset, record['postal_code'] = get_data(buf, offset + 1)
-        buf_pos = offset + 1
+        offset, record['region_code'] = read_data(buf, 1)
+        offset, record['city'] = read_data(buf, offset + 1)
+        offset, record['postal_code'] = read_data(buf, offset + 1)
+        offset = offset + 1
 
         for j in range(3):
-            char = ord(buf[buf_pos])
-            buf_pos += 1
-            latitude += (char << (j * 8))
+            latitude += (ord(buf[offset + j]) << (j * 8))
 
         for j in range(3):
-            char = ord(buf[buf_pos])
-            buf_pos += 1
-            longitude += (char << (j * 8))
+            longitude += (ord(buf[offset + j + 3]) << (j * 8))
 
         record['latitude'] = (latitude / 10000.0) - 180.0
         record['longitude'] = (longitude / 10000.0) - 180.0
 
         if self._databaseType in (const.CITY_EDITION_REV1, const.CITY_EDITION_REV1_V6):
-            dmaarea_combo = 0
             if record['country_code'] == 'US':
+                dma_area = 0
                 for j in range(3):
-                    char = ord(buf[buf_pos])
-                    dmaarea_combo += (char << (j * 8))
-                    buf_pos += 1
+                    dma_area += ord(buf[offset + j + 6]) << (j * 8)
 
-                record['dma_code'] = int(math.floor(dmaarea_combo / 1000))
-                record['area_code'] = dmaarea_combo % 1000
+                record['dma_code'] = int(floor(dma_area / 1000))
+                record['area_code'] = dma_area % 1000
+                record['metro_code'] = const.DMA_MAP.get(record['dma_code'])
 
-        record['metro_code'] = const.DMA_MAP.get(record['dma_code'])
-        params = (record['country_code'], record['region_name'])
+        params = (record['country_code'], record['region_code'])
         record['time_zone'] = time_zone_by_country_and_region(*params)
 
         return record
 
     def _gethostbyname(self, hostname):
         if self._databaseType in const.IPV6_EDITIONS:
-            try:
-                response = socket.getaddrinfo(hostname, 0, socket.AF_INET6)
-                family, socktype, proto, canonname, sockaddr = response[0]
-                address, port, flow, scope = sockaddr
-                return address
-            except socket.gaierror:
-                return ''
+            response = socket.getaddrinfo(hostname, 0, socket.AF_INET6)
+            family, socktype, proto, canonname, sockaddr = response[0]
+            address, port, flow, scope = sockaddr
+            return address
         else:
             return socket.gethostbyname(hostname)
 
-    def id_by_addr(self, addr):
+    def _id_by_addr(self, addr):
         """
-        Get the country index.
-        Looks up the index for the country which is the key for
-        the code and name.
+        Looks up the index for the country which is the key for the
+        code and name.
 
-        @param addr: The IP address
+        @param addr: IPv4 or IPv6 address
         @type addr: str
         @return: network byte order 32-bit integer
         @rtype: int
         """
+        ipv = 6 if addr.find(':') >= 0 else 4
+        if ipv == 4 and self._databaseType != const.COUNTRY_EDITION:
+            raise GeoIPError('Invalid database type; expected IPv6 address')
+        if ipv == 6 and self._databaseType != const.COUNTRY_EDITION_V6:
+            raise GeoIPError('Invalid database type; expected IPv4 address')
+
         ipnum = util.ip2long(addr)
-        if not ipnum:
-            raise ValueError("Invalid IP address: %s" % addr)
-
-        COUNTY_EDITIONS = (const.COUNTRY_EDITION, const.COUNTRY_EDITION_V6)
-        if self._databaseType not in COUNTY_EDITIONS:
-            message = 'Invalid database type, expected Country'
-            raise GeoIPError(message)
-
         return self._seek_country(ipnum) - const.COUNTRY_BEGIN
 
+    def last_netmask(self):
+        """
+        Return the netmask depth of the last lookup.
+
+        @return: network depth
+        @rtype: int
+        """
+        return self._netmask
+
     def country_code_by_addr(self, addr):
         """
         Returns 2-letter country code (e.g. 'US') for specified IP address.
@@ -443,27 +458,14 @@ class GeoIP(GeoIPBase):
         @return: 2-letter country code
         @rtype: str
         """
-        try:
-            VALID_EDITIONS = (const.COUNTRY_EDITION, const.COUNTRY_EDITION_V6)
-            if self._databaseType in VALID_EDITIONS:
-                ipv = 6 if addr.find(':') >= 0 else 4
+        VALID_EDITIONS = (const.COUNTRY_EDITION, const.COUNTRY_EDITION_V6)
+        if self._databaseType in VALID_EDITIONS:
+            country_id = self._id_by_addr(addr)
+            return const.COUNTRY_CODES[country_id]
+        elif self._databaseType in const.REGION_CITY_EDITIONS:
+            return self.region_by_addr(addr).get('country_code')
 
-                if ipv == 4 and self._databaseType != const.COUNTRY_EDITION:
-                    message = 'Invalid database type; expected IPv6 address'
-                    raise ValueError(message)
-                if ipv == 6 and self._databaseType != const.COUNTRY_EDITION_V6:
-                    message = 'Invalid database type; expected IPv4 address'
-                    raise ValueError(message)
-
-                country_id = self.id_by_addr(addr)
-                return const.COUNTRY_CODES[country_id]
-            elif self._databaseType in const.REGION_CITY_EDITIONS:
-                return self.region_by_addr(addr).get('country_code')
-
-            message = 'Invalid database type, expected Country, City or Region'
-            raise GeoIPError(message)
-        except ValueError:
-            raise GeoIPError('Failed to lookup address %s' % addr)
+        raise GeoIPError('Invalid database type, expected Country, City or Region')
 
     def country_code_by_name(self, hostname):
         """
@@ -488,18 +490,15 @@ class GeoIP(GeoIPBase):
         @return: country name
         @rtype: str
         """
-        try:
-            VALID_EDITIONS = (const.COUNTRY_EDITION, const.COUNTRY_EDITION_V6)
-            if self._databaseType in VALID_EDITIONS:
-                country_id = self.id_by_addr(addr)
-                return const.COUNTRY_NAMES[country_id]
-            elif self._databaseType in const.CITY_EDITIONS:
-                return self.record_by_addr(addr).get('country_name')
-            else:
-                message = 'Invalid database type, expected Country or City'
-                raise GeoIPError(message)
-        except ValueError:
-            raise GeoIPError('Failed to lookup address %s' % addr)
+        VALID_EDITIONS = (const.COUNTRY_EDITION, const.COUNTRY_EDITION_V6)
+        if self._databaseType in VALID_EDITIONS:
+            country_id = self._id_by_addr(addr)
+            return const.COUNTRY_NAMES[country_id]
+        elif self._databaseType in const.CITY_EDITIONS:
+            return self.record_by_addr(addr).get('country_name')
+        else:
+            message = 'Invalid database type, expected Country or City'
+            raise GeoIPError(message)
 
     def country_name_by_name(self, hostname):
         """
@@ -524,19 +523,13 @@ class GeoIP(GeoIPBase):
         @return: organization or ISP name
         @rtype: str
         """
-        try:
-            ipnum = util.ip2long(addr)
-            if not ipnum:
-                raise ValueError('Invalid IP address')
+        valid = (const.ORG_EDITION, const.ISP_EDITION, const.ASNUM_EDITION, const.ASNUM_EDITION_V6)
+        if self._databaseType not in valid:
+            message = 'Invalid database type, expected Org, ISP or ASNum'
+            raise GeoIPError(message)
 
-            valid = (const.ORG_EDITION, const.ISP_EDITION, const.ASNUM_EDITION, const.ASNUM_EDITION_V6)
-            if self._databaseType not in valid:
-                message = 'Invalid database type, expected Org, ISP or ASNum'
-                raise GeoIPError(message)
-
-            return self._get_org(ipnum)
-        except ValueError:
-            raise GeoIPError('Failed to lookup address %s' % addr)
+        ipnum = util.ip2long(addr)
+        return self._get_org(ipnum)
 
     def org_by_name(self, hostname):
         """
@@ -560,25 +553,19 @@ class GeoIP(GeoIPBase):
         @type addr: str
         @return: Dictionary with country_code, country_code3, country_name,
             region, city, postal_code, latitude, longitude, dma_code,
-            metro_code, area_code, region_name, time_zone
+            metro_code, area_code, region_code, time_zone
         @rtype: dict
         """
-        try:
-            ipnum = util.ip2long(addr)
-            if not ipnum:
-                raise ValueError('Invalid IP address')
+        if self._databaseType not in const.CITY_EDITIONS:
+            message = 'Invalid database type, expected City'
+            raise GeoIPError(message)
 
-            if self._databaseType not in const.CITY_EDITIONS:
-                message = 'Invalid database type, expected City'
-                raise GeoIPError(message)
+        ipnum = util.ip2long(addr)
+        rec = self._get_record(ipnum)
+        if not rec:
+            return None
 
-            rec = self._get_record(ipnum)
-            if not rec:
-                return None
-
-            return rec
-        except ValueError:
-            raise GeoIPError('Failed to lookup address %s' % addr)
+        return rec
 
     def record_by_name(self, hostname):
         """
@@ -589,7 +576,7 @@ class GeoIP(GeoIPBase):
         @type hostname: str
         @return: Dictionary with country_code, country_code3, country_name,
             region, city, postal_code, latitude, longitude, dma_code,
-            metro_code, area_code, region_name, time_zone
+            metro_code, area_code, region_code, time_zone
         @rtype: dict
         """
         addr = self._gethostbyname(hostname)
@@ -602,21 +589,15 @@ class GeoIP(GeoIPBase):
 
         @param addr: IP address
         @type addr: str
-        @return: Dictionary containing country_code, region and region_name
+        @return: Dictionary containing country_code and region_code
         @rtype: dict
         """
-        try:
-            ipnum = util.ip2long(addr)
-            if not ipnum:
-                raise ValueError('Invalid IP address')
+        if self._databaseType not in const.REGION_CITY_EDITIONS:
+            message = 'Invalid database type, expected Region or City'
+            raise GeoIPError(message)
 
-            if self._databaseType not in const.REGION_CITY_EDITIONS:
-                message = 'Invalid database type, expected Region or City'
-                raise GeoIPError(message)
-
-            return self._get_region(ipnum)
-        except ValueError:
-            raise GeoIPError('Failed to lookup address %s' % addr)
+        ipnum = util.ip2long(addr)
+        return self._get_region(ipnum)
 
     def region_by_name(self, hostname):
         """
@@ -625,7 +606,7 @@ class GeoIP(GeoIPBase):
 
         @param hostname: Hostname
         @type hostname: str
-        @return: Dictionary containing country_code, region, and region_name
+        @return: Dictionary containing country_code, region_code and region
         @rtype: dict
         """
         addr = self._gethostbyname(hostname)
@@ -641,18 +622,12 @@ class GeoIP(GeoIPBase):
         @return: Time zone
         @rtype: str
         """
-        try:
-            ipnum = util.ip2long(addr)
-            if not ipnum:
-                raise ValueError('Invalid IP address')
+        if self._databaseType not in const.CITY_EDITIONS:
+            message = 'Invalid database type, expected City'
+            raise GeoIPError(message)
 
-            if self._databaseType not in const.CITY_EDITIONS:
-                message = 'Invalid database type, expected City'
-                raise GeoIPError(message)
-
-            return self._get_record(ipnum).get('time_zone')
-        except ValueError:
-            raise GeoIPError('Failed to lookup address %s' % addr)
+        ipnum = util.ip2long(addr)
+        return self._get_record(ipnum).get('time_zone')
 
     def time_zone_by_name(self, hostname):
         """
