@@ -1,8 +1,9 @@
 # -*- coding: utf-8 -*-
 """
-Time zone functions. Part of the pygeoip package.
+Time zone data and lookup function
 
 @author: Jennifer Ennis <zaylea@gmail.com>
+@author: William Tisäter <william@defunct.cc>
 
 @license: Copyright(C) 2004 MaxMind LLC
 
@@ -20,9 +21,29 @@ You should have received a copy of the GNU Lesser General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/lgpl.txt>.
 """
 
-__all__ = ['time_zone_by_country_and_region']
 
-_country = {
+def time_zone_by_country_and_region(country_code, region_code=None):
+    """
+    Get time zone from country code and region code.
+
+    @param country_code: Country code
+    @type country_code: str
+    @param region_code: Region code
+    @type region_code: str
+    @return: Time zone
+    @rtype: str
+    """
+    timezone = country_dict.get(country_code)
+    if not timezone:
+        return None
+
+    if isinstance(timezone, str):
+        return timezone
+
+    return timezone.get(region_code)
+
+
+country_dict = {
     'AD': 'Europe/Andorra',
     'AE': 'Asia/Dubai',
     'AF': 'Asia/Kabul',
@@ -740,21 +761,4 @@ _country = {
     'ZA': 'Africa/Johannesburg',
     'ZM': 'Africa/Lusaka',
     'ZW': 'Africa/Harare'
- }
-
-
-def time_zone_by_country_and_region(country_code, region_name=None):
-    if country_code not in _country:
-        return ''
-
-    if not region_name or region_name == '00':
-        region_name = None
-
-    timezones = _country[country_code]
-    if isinstance(timezones, str):
-        return timezones
-
-    if not region_name:
-        return ''
-
-    return timezones.get(region_name)
+}
