@@ -1,10 +1,11 @@
+# -*- coding: utf-8 -*-
 """
-Misc. utility functions. It is part of the pygeoip package.
+Utility function for address translation
 
-@author: Jennifer Ennis <zaylea at gmail dot com>
+@author: Jennifer Ennis <zaylea@gmail.com>
+@author: William Tisäter <william@defunct.cc>
 
-@license:
-Copyright(C) 2004 MaxMind LLC
+@license: Copyright(C) 2004 MaxMind LLC
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
@@ -20,16 +21,17 @@ You should have received a copy of the GNU Lesser General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/lgpl.txt>.
 """
 
+import socket
+import binascii
+
+
 def ip2long(ip):
     """
-    Convert a IPv4 address into a 32-bit integer.
-    
-    @param ip: quad-dotted IPv4 address
+    Wrapper function for IPv4 and IPv6 converters
+    @param ip: IPv4 or IPv6 address
     @type ip: str
-    @return: network byte order 32-bit integer
-    @rtype: int
     """
-    ip_array = ip.split('.')
-    ip_long = long(ip_array[0]) * 16777216 + long(ip_array[1]) * 65536 + long(ip_array[2]) * 256 + long(ip_array[3])
-    return ip_long  
-
+    try:
+        return int(binascii.hexlify(socket.inet_aton(ip)), 16)
+    except socket.error:
+        return int(binascii.hexlify(socket.inet_pton(socket.AF_INET6, ip)), 16)
